
#include "BindObjectToName.h"

#include <iostream>
using std::cerr;
using std::endl;

using std::string;

bool BindObjectToName(
   CORBA::ORB_ptr orb, 
   CORBA::Object_ptr objref,
   const string &contextName, 
   const string &objectName)
{
   bool ok = false;

   CosNaming::NamingContext_var rootContext;

   try 
   {
      CORBA::Object_var obj = orb->resolve_initial_references("NameService");

      rootContext = CosNaming::NamingContext::_narrow(obj);

      if( CORBA::is_nil(rootContext) ) 
      {
         cerr << "Failed to narrow the root naming context." << endl;
         return 0;
      }
   }
   catch(CORBA::ORB::InvalidName& ) 
   {
      cerr << "Service required is invalid [does not exist]." << endl;
      return 0;
   }


   try 
   {
      CosNaming::Name context_name;
      context_name.length(1);
      context_name[0].id   = contextName.c_str();
      context_name[0].kind = (const char*) "JetByte Context"; 

      CosNaming::NamingContext_var testContext;
 
      try 
      {
         testContext = rootContext->bind_new_context(context_name);
      }
      catch(CosNaming::NamingContext::AlreadyBound&) 
      {
         CORBA::Object_var obj = rootContext->resolve(context_name);
      
         testContext = CosNaming::NamingContext::_narrow(obj);

         if(CORBA::is_nil(testContext)) 
         {
            cerr << "Failed to narrow naming context." << endl;
            return false;
         }
      }


      // Bind the objref to the name in the testContext:

      CosNaming::Name object_name;
      object_name.length(1);
      object_name[0].id   = objectName.c_str();
      object_name[0].kind = (const char*) "Object"; 

      try 
      {
         testContext->bind(object_name, objref);
         ok = true;
      }
      catch(CosNaming::NamingContext::AlreadyBound &) 
      {
         testContext->rebind(object_name, objref);
         ok = true;
      }
   }
   catch(CORBA::COMM_FAILURE &) 
   {
      cerr << "System exception: COMM_FAILURE" << endl;
   }
   catch(CORBA::SystemException &) 
   {
      cerr << "System exception:" << endl;
   }

   return ok;
}
